using Exam.Infrastructure.Extensions;
using Exam.Infrastructure.Validation.Validation;
using FluentValidation;
using Hotline.Exams.ExamManages;
using Hotline.Exams.Questions;
using Hotline.Exams.Validate;
using Hotline.KnowledgeBase;
using Hotline.Repository.SqlSugar.Exam.Validate;
using ExamQuestion = Hotline.Exams.Questions.ExamQuestion;
using ExamQuestionKnowladge = Hotline.Exams.Questions.ExamQuestionKnowladge;

namespace Exam.Repository.Sqlsugar.Validators.ExamManages
{
    public class ExamQuestionKnowladgeValidator:BaseValidator<Hotline.Exams.ExamManages.ExamQuestionKnowladgeBak>
    {
        public ExamQuestionKnowladgeValidator()
        {
            RuleSet(ValidatorTypeConstants.Create, () =>
            {
                BaseValidateRule();

                ValidateRuleWithAdd();
            });

            RuleSet(ValidatorTypeConstants.Modify, () =>
            {
                BaseValidateRule();

                ValidateRuleWithModify();
            });
        }

        protected override void BaseValidateRule()
        {
            base.BaseValidateRule();
            RuleFor(m => m.KnowladgeId).NotEmpty().WithMessage(x => string.Format(ExamErrorMessage.IsRequired, typeof(Knowledge).GetDescription()));
            RuleFor(m => m.QuestionId).NotEmpty().WithMessage(x => string.Format(ExamErrorMessage.IsRequired, typeof(ExamQuestion).GetDescription()));
        }

        protected override void ValidateRuleWithAdd()
        {
            base.ValidateRuleWithAdd();
            RuleFor(m => m.CreationTime).NotEmpty().WithMessage(x => string.Format(ExamErrorMessage.IsRequired, x.GetType().GetDescription(nameof(ExamQuestionKnowladge.CreationTime))));
        }

        protected override void ValidateRuleWithModify()
        {
            base.ValidateRuleWithModify();
            RuleFor(m => m.LastModificationTime).NotEmpty().WithMessage(x => string.Format(ExamErrorMessage.IsRequired, x.GetType().GetDescription(nameof(ExamQuestionKnowladge.LastModificationTime))));

        }

    }
}
