﻿using Hotline.Share.Dtos.Snapshot;
using Hotline.Snapshot.Contracts;
using Hotline.Snapshot.IRepository;
using Mapster;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using XF.Domain.Dependency;

namespace Hotline.Snapshot.Services;

public class NotificationDomainService : INotificationDomainService, IScopeDependency
{
    private readonly INotificationRepository _notificationRepository;
    private readonly INotificationReceiverRepository _notificationReceiverRepository;

    public NotificationDomainService(INotificationRepository notificationRepository, INotificationReceiverRepository notificationReceiverRepository)
    {
        _notificationRepository = notificationRepository;
        _notificationReceiverRepository = notificationReceiverRepository;
    }

    public async Task AddNotifyAsync(AddNotifyInDto inDto, CancellationToken token)
    {
        var entity = inDto.Adapt<Notification>();
        var id = await _notificationRepository.AddAsync(entity, token);
        var items = new List<NotificationReceiver>();
        foreach (var userId in inDto.UserIds)
        {
            items.Add(new NotificationReceiver { 
                NotificationId = id,
                ReceiverId = userId,
                IsRead = false
            });
        }
        await _notificationReceiverRepository.AddRangeAsync(items, token);
    }
}
